/*-------------------------------------------------------------------------
 *
 * FILE
 *    in_out.c
 *
 * DESCRIPTION
 *   UUID type input and output routine
 *
 * Copyright (c) 2002-2003, Mike Wang <xylophone2001@hotmail.com>
 *
 *-------------------------------------------------------------------------
 *
 * $Header: /usr/local/cvsroot/pguuid/pguuid/src/in_out.c,v 1.4 2003/06/11 05:42:12 xylophone Exp $
 *
 *-------------------------------------------------------------------------
 */
#include <uuid/uuid.h>

#include <postgres.h>
#include <fmgr.h>


typedef unsigned char * uuid_p;


/* in func */
PG_FUNCTION_INFO_V1(uuid_in);
Datum uuid_in(PG_FUNCTION_ARGS)
{
	uuid_p result;

	const char* in = PG_GETARG_CSTRING(0);
	result = (uuid_p)palloc(16);
	char *buf;

	switch (strlen(in)) {
	case 36:
	   break;
	case 38:
	   if (in[0]!='{' || in[37]!='}') {
	      elog(ERROR, "38-char UUID must be {}-wrapped");
	      PG_RETURN_POINTER(NULL);
	   } 
	   buf = alloca(37);
	   strncpy(buf,in+1,36);
	   buf[36] = 0;
	   in = buf;
	   break;
	case 32:
	   buf = alloca(37);
	   strncpy(buf+ 0,in+ 0,8);
	   buf[8]='-';
	   strncpy(buf+ 9,in+ 8,4);
	   buf[13]='-';
	   strncpy(buf+14,in+12,4);
	   buf[18]='-';
	   strncpy(buf+19,in+16,4);
	   buf[23]='-';
	   strcpy(buf+24,in+20);
	   in = buf;
	   break;
	default:
	   elog(ERROR, "UUID must be given as 32,36 or 38 chars");
	   PG_RETURN_POINTER(NULL);
	}

	if(uuid_parse(in, result) < 0)
	{
	   elog(ERROR, "%s is invalid UUID format!", in);
	   PG_RETURN_POINTER(NULL);
	}

	PG_RETURN_POINTER(result);
}

/* in text func */
PG_FUNCTION_INFO_V1(uuid_in_text);
Datum uuid_in_text(PG_FUNCTION_ARGS)
{
	uuid_p result;

	text *textIn = PG_GETARG_TEXT_P(0);
	result = (uuid_p)palloc(16);
	char *in = VARDATA(textIn); /* not NUL-terminated! */
	char *buf = alloca(37);
	int len = VARSIZE(textIn) - VARHDRSZ;

	switch (len) {
	case 36:
	   strncpy(buf,in,len);
	   break;
	case 38:
	   if (in[0]!='{' || in[37]!='}') {
	      elog(ERROR, "38-char UUID must be {}-wrapped");
	      PG_RETURN_POINTER(NULL);
	   } 
	   strncpy(buf,in+1,36);
	   break;
	case 32:
	   strncpy(buf+ 0,in+ 0,8);
	   buf[8]='-';
	   strncpy(buf+ 9,in+ 8,4);
	   buf[13]='-';
	   strncpy(buf+14,in+12,4);
	   buf[18]='-';
	   strncpy(buf+19,in+16,4);
	   buf[23]='-';
	   strncpy(buf+24,in+20,12);
	   break;
	default:
	   elog(ERROR, "UUID must be given as 32,36 or 38 chars");
	   PG_RETURN_POINTER(NULL);
	}
	buf[36] = 0;

	if(uuid_parse(in, result) < 0)
	{
	   elog(ERROR, "invalid UUID format!");
	   PG_RETURN_POINTER(NULL);
	}

	PG_RETURN_POINTER(result);
}




/* out func */
PG_FUNCTION_INFO_V1(uuid_out);
Datum uuid_out(PG_FUNCTION_ARGS)
{
	char* result = 0;

	uuid_p ptr = (uuid_p) PG_GETARG_POINTER(0);
	result = (char*)palloc(sizeof(char) * 36);

	uuid_unparse(ptr, result);

	PG_RETURN_CSTRING(result);
}

/* out text cast func */
PG_FUNCTION_INFO_V1(uuid_out_text);
Datum uuid_out_text(PG_FUNCTION_ARGS)
{
   uuid_p ptr = (uuid_p) PG_GETARG_POINTER(0);
   text *result = (text*) palloc(37+VARHDRSZ);
   
   uuid_unparse(ptr, VARDATA(result));
   VARATT_SIZEP(result) = 36+VARHDRSZ;
   PG_RETURN_TEXT_P(result);
}
