/*-------------------------------------------------------------------------
 *
 * FILE
 *    newid.c
 *
 * DESCRIPTION
 *    generate a new UUID for PosgreSQL
 *
 * Copyright (c) 2002-2003, Mike Wang <xylophone2001@hotmail.com>
 * 
 *
 *-------------------------------------------------------------------------
 *
 * different uuid type implement method
 *
 * $Header: /usr/local/cvsroot/pguuid/pguuid/src/newid.c,v 1.5 2003/06/11 05:43:24 xylophone Exp $
 *
 *-------------------------------------------------------------------------
 */
#include <uuid/uuid.h>

#include <postgres.h>
#include <fmgr.h>


typedef unsigned char * uuid_p;


static const char * newid_copyright_string
= "Copyright (C) 2008 David Lee Lambert"
"\n Based on pguuid by Mike Wang";


#if PGLEVEL==82
PG_MODULE_MAGIC;
#endif




/** Random UUID */
PG_FUNCTION_INFO_V1(newid_random);
Datum newid_random(PG_FUNCTION_ARGS)
{
	uuid_p result;

	result = (uuid_p) palloc(16);

	uuid_generate_random(result);

	PG_RETURN_POINTER(result);
}

/** Time-based UUID */
PG_FUNCTION_INFO_V1(newid_time);
Datum newid_time(PG_FUNCTION_ARGS)
{
	uuid_p result;

	result = (uuid_p)palloc(16);

	uuid_generate_time(result);

	PG_RETURN_POINTER(result);
}

/** UUID by the default algorithm.
 *  Stock libuuid prefers random and falls back to time-based.  
 *     */
PG_FUNCTION_INFO_V1(newid_default);
Datum newid_default(PG_FUNCTION_ARGS)
{
	uuid_p result;

	result = (uuid_p)palloc(16);

	uuid_generate(result);

	PG_RETURN_POINTER(result);
}

/* NIL UUID */
PG_FUNCTION_INFO_V1(newid_nil);
Datum newid_nil(PG_FUNCTION_ARGS)
{
	uuid_p result;

	result = (uuid_p) palloc(16);

	uuid_clear(result);

	PG_RETURN_POINTER(result);
}

/*
... leaving out the name-based generators for now
*/
