/*-------------------------------------------------------------------------
 *
 * FILE
 *    in_out.c
 *
 * DESCRIPTION
 *   UUID type input and output routine
 *
 * Copyright (c) 2002-2003, Mike Wang <xylophone2001@hotmail.com>
 *
 *-------------------------------------------------------------------------
 *
 * $Header: /usr/local/cvsroot/pguuid/pguuid/src/in_out.c,v 1.4 2003/06/11 05:42:12 xylophone Exp $
 *
 *-------------------------------------------------------------------------
 */
#include <uuid/uuid.h>

#include <postgres.h>
#include <fmgr.h>


typedef unsigned char * uuid_p;



static uuid_p nice_uuid_parse(const char* in,int len,uuid_p result) {
   char *buf = alloca(37);
   buf[36] = 0;
   switch (len) {
   case 36:
      memcpy(buf, in, 36);
      break;
   case 38:
      if (in[0]!='{' || in[37]!='}') {
	 elog(ERROR, "38-char UUID must be {}-wrapped");
	 return NULL;
      } 
      buf = alloca(37);
      strncpy(buf,in+1,36);
      break;
   case 32:
      strncpy(buf+ 0,in+ 0,8);
      buf[8]='-';
      strncpy(buf+ 9,in+ 8,4);
      buf[13]='-';
      strncpy(buf+14,in+12,4);
      buf[18]='-';
      strncpy(buf+19,in+16,4);
      buf[23]='-';
      strncpy(buf+24,in+20,12);
      break;
   default:
      elog(ERROR, "UUID must be given as 32,36 or 38 chars");
      return NULL;
   }
   
   if (uuid_parse(in, result) < 0) {
      buf = alloca(55);
      strcpy(buf+50,"...");
      strncpy(buf,in,len>50?50:len);
      elog(ERROR, "invalid UUID format '%s'",buf);
   }
    
   return result;
}


/* in func */
PG_FUNCTION_INFO_V1(uuid_in);
Datum uuid_in(PG_FUNCTION_ARGS)
{
	uuid_p result;

	const char* in = PG_GETARG_CSTRING(0);
	result = (uuid_p)palloc(16);
	char *buf;

	int len = strlen(in);
	if (nice_uuid_parse(in,len,result)==NULL) {
	   pfree(result);
	   PG_RETURN_POINTER(NULL);
	}

	PG_RETURN_POINTER(result);
}

/* in text cast func */
PG_FUNCTION_INFO_V1(uuid_in_text);
Datum uuid_in_text(PG_FUNCTION_ARGS)
{
   uuid_p result = palloc(16);
   text *in = PG_GETARG_TEXT_P(0);
   int len  = VARSIZE(in) - VARHDRSZ;
   char *str = VARDATA(in);

   while (len>0 && str[len]==' '||str[len]==0)
      --len;
	
	if (nice_uuid_parse(str,len,result)==NULL) {
	   pfree(result);
	   PG_RETURN_POINTER(NULL);
	}

	PG_RETURN_POINTER(result);
}


/* out func */
PG_FUNCTION_INFO_V1(uuid_out);
Datum uuid_out(PG_FUNCTION_ARGS)
{
	char* result = 0;

	uuid_p ptr = (uuid_p) PG_GETARG_POINTER(0);
	result = (char*)palloc(sizeof(char) * 36);

	uuid_unparse(ptr, result);

	PG_RETURN_CSTRING(result);
}

/* out text cast func */
PG_FUNCTION_INFO_V1(uuid_out_text);
Datum uuid_out_text(PG_FUNCTION_ARGS)
{
   uuid_p ptr = (uuid_p) PG_GETARG_POINTER(0);
   text *result = (text*) palloc(37+VARHDRSZ);
   
   uuid_unparse(ptr, VARDATA(result));
   VARSIZE(result) = 36+VARHDRSZ;
   PG_RETURN_TEXT_P(result);
}


  
