/*-------------------------------------------------------------------------
 *
 * FILE
 *    newid.c
 *
 * DESCRIPTION
 *    generate a new UUID for PosgreSQL
 *
 * Copyright (c) 2002-2003, Mike Wang <xylophone2001@hotmail.com>
 * Copyright (c) 2009, David Lee Lambert <davidl@lmert.com>
 *
 *-------------------------------------------------------------------------
 *
 * different uuid type implement method
 *
 * $Id: newid.c 284 2009-02-08 04:00:43Z davidl $
 *-------------------------------------------------------------------------
 */
#include <uuid/uuid.h>

#include <postgres.h>
#include <fmgr.h>

#include "access/hash.h"
#include "libpq/pqformat.h"
#include "utils/builtins.h"
#include "utils/uuid.h"


typedef unsigned char * uuid_p;

/* uuid size in bytes */
#define UUID_LEN 16

/* pg_uuid_t is declared to be struct pg_uuid_t in uuid.h */
struct pg_uuid_t
{
	unsigned char data[UUID_LEN];
};

static const char * newid_copyright_string
= "Copyright (C) 2009 David Lee Lambert"
"\n Based on pguuid by Mike Wang";



PG_MODULE_MAGIC;



/** Random UUID */
PG_FUNCTION_INFO_V1(newid_random);
Datum newid_random(PG_FUNCTION_ARGS)
{
	pg_uuid_t  *uuid;
	uuid = (pg_uuid_t *) palloc(sizeof(*uuid));

	uuid_generate_random((uuid_p) &(uuid->data));

	PG_RETURN_UUID_P(uuid);
}

/** Time-based UUID */
PG_FUNCTION_INFO_V1(newid_time);
Datum newid_time(PG_FUNCTION_ARGS)
{
	pg_uuid_t  *uuid;
	uuid = (pg_uuid_t *) palloc(sizeof(*uuid));

	uuid_generate_time((uuid_p) &(uuid->data));

	PG_RETURN_UUID_P(uuid);
}

/** UUID by the default algorithm.
 *  Stock libuuid prefers random and falls back to time-based.  
 *     */
PG_FUNCTION_INFO_V1(newid_default);
Datum newid_default(PG_FUNCTION_ARGS)
{
	pg_uuid_t  *uuid;
	uuid = (pg_uuid_t *) palloc(sizeof(*uuid));

	uuid_generate((uuid_p) &(uuid->data));

	PG_RETURN_UUID_P(uuid);
}

/* NIL UUID */
PG_FUNCTION_INFO_V1(newid_nil);
Datum newid_nil(PG_FUNCTION_ARGS)
{
	pg_uuid_t  *uuid;
	uuid = (pg_uuid_t *) palloc(sizeof(*uuid));

	uuid_clear((uuid_p) &(uuid->data));

	PG_RETURN_UUID_P(uuid);
}

/*
... leaving out the name-based generators for now
*/
